#!/usr/bin/env python

import pyglet
from pyglet.gl import *

buildingTextures = [pyglet.image.load("game/tilebricks.jpg").get_mipmapped_texture()]

for t in buildingTextures:
    glBindTexture(t.target, t.id)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT)
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT)


class Building(object):
    def __init__(self, dimdict):
        self.xyz = (dimdict['x'], dimdict['y'], 0)
        self.color = [float(dimdict['depth'])/255.0, float(dimdict['depth'])/255.0, float(dimdict['depth'])/255.0]
        
        self.dims = (dimdict['width'], dimdict['height'], -dimdict['depth'])
        self.floorverts = [list(self.xyz), 
                            [self.xyz[0] + self.dims[0], self.xyz[1], self.xyz[2]],
                            [self.xyz[0] + self.dims[0], self.xyz[1] + self.dims[1], self.xyz[2]],
                            [self.xyz[0], self.xyz[1] + self.dims[1], self.xyz[2]]
                          ]
        self.roofverts = list()
        for x in self.floorverts:
            self.roofverts.append(x[:])
            self.roofverts[-1][2] = dimdict['depth']
            
        self.texture = buildingTextures[0]
        
    def draw(self, collisionCallback, hothouseCallback, coordinates):
        
        # it's dirty that I placed this in the draw function, I know.
        if coordinates[2] <= self.roofverts[0][2]:
            if coordinates[0] >= self.roofverts[0][0] and coordinates[0] <= self.roofverts[1][0]:
                collisionCallback(self)
            elif coordinates[1] >= self.roofverts[0][1] and coordinates[1] <= self.roofverts[2][1]:
                collisionCallback(self)
            
        glColor3f(*self.color)
        glBindTexture(self.texture.target, self.texture.id)
        # draw the roof
        glBegin(GL_QUADS)
        glTexCoord3f(0.0, 0.0, 0.0)
        glVertex3f(*self.roofverts[0])
        glTexCoord3f(1.0, 0.0, 0.0)
        glVertex3f(*self.roofverts[1])
        glTexCoord3f(1.0, 1.0, 0.0)
        glVertex3f(*self.roofverts[2])
        glTexCoord3f(0.0, 1.0, 0.0)
        glVertex3f(*self.roofverts[3])
        
        glTexCoord3f(0.0, 0.0, 0.0)
        glVertex3f(*self.floorverts[0])
        glTexCoord3f(1.0, 0.0, 0.0)
        glVertex3f(*self.floorverts[1])
        glTexCoord3f(1.0, 1.0, 0.0)
        glVertex3f(*self.roofverts[1])
        glTexCoord3f(0.0, 1.0, 0.0)
        glVertex3f(*self.roofverts[0])
        
        glTexCoord3f(0.0, 0.0, 0.0)
        glVertex3f(*self.floorverts[1])
        glTexCoord3f(1.0, 0.0, 0.0)
        glVertex3f(*self.floorverts[2])
        glTexCoord3f(1.0, 1.0, 0.0)
        glVertex3f(*self.roofverts[2])
        glTexCoord3f(0.0, 1.0, 0.0)
        glVertex3f(*self.roofverts[1])

        glTexCoord3f(0.0, 0.0, 0.0)
        glVertex3f(*self.floorverts[2])
        glTexCoord3f(1.0, 0.0, 0.0)
        glVertex3f(*self.floorverts[3])
        glTexCoord3f(1.0, 1.0, 0.0)
        glVertex3f(*self.roofverts[3])
        glTexCoord3f(0.0, 1.0, 0.0)
        glVertex3f(*self.roofverts[2])

        glTexCoord3f(0.0, 0.0, 0.0)
        glVertex3f(*self.floorverts[3])
        glTexCoord3f(1.0, 0.0, 0.0)
        glVertex3f(*self.floorverts[0])
        glTexCoord3f(1.0, 1.0, 0.0)
        glVertex3f(*self.roofverts[0])
        glTexCoord3f(0.0, 1.0, 0.0)
        glVertex3f(*self.roofverts[3])
        glEnd()
        glBindTexture(self.texture.target, 0)