#!/usr/bin/env python
from pyglet.gl import *

# a python translation of http://www.flipcode.com/archives/OpenGL_Camera.shtml
    
class Camera(object):
    def __init__(self, x, y, z):
        
        self.transform = (GLfloat * 16)(1.0, 0.0, 0.0, 0.0,
                          0.0, 1.0, 0.0, 0.0,
                          0.0, 0.0, -1.0, 0.0,
                          x, y, z, 1.0)

        self.right    = self.transform[0:4]
        self.up       = self.transform[4:8]
        self.forward  = self.transform[8:12]
        self.position = self.transform[12:]



    # This one does pretty much the same as gluLookAt, just that it doesnt require
    # to extract the vectors for gluLookAt and have it rebuild the matrix we already
    # got.
    def setView(self):
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        viewmatrix = (GLfloat * 16)(
                      self.transform[0], self.transform[4], self.transform[8], 0,
                      self.transform[1], self.transform[5], self.transform[9], 0,
                      self.transform[2], self.transform[6], self.transform[10], 0,

                      -(self.transform[0]*self.transform[12] +
                      self.transform[1]*self.transform[13] +
                      self.transform[2]*self.transform[14]),

                      -(self.transform[4]*self.transform[12] +
                      self.transform[5]*self.transform[13] +
                      self.transform[6]*self.transform[14]),

                      -(self.transform[8]*self.transform[12] +
                      self.transform[9]*self.transform[13] +
                      self.transform[10]*self.transform[14]), 1)
        glLoadMatrixf(viewmatrix)

    def moveLoc(self, x, y, z, distance):
        dx=x*self.transform[0] + y*self.transform[4] + z*self.transform[8]
        dy=x*self.transform[1] + y*self.transform[5] + z*self.transform[9]
        dz=x*self.transform[2] + y*self.transform[6] + z*self.transform[10]
        self.transform[12] += dx * distance
        self.transform[13] += dy * distance
        self.transform[14] += dz * distance

    def moveGlob(self, x, y, z, distance):
        self.transform[12] += x * distance
        self.transform[13] += y * distance
        self.transform[14] += z * distance

    # Here we let OpenGls (most likely quite optimized) functions do the work.
    # Note that its transformations already are in local coords.
    def rotateLoc(self, deg, x, y, z):
        glMatrixMode(GL_MODELVIEW)
        glPushMatrix()
        glLoadMatrixf(self.transform)
        glRotatef(deg, x,y,z)
        glGetFloatv(GL_MODELVIEW_MATRIX, self.transform)
        glPopMatrix()

    # We have to invert the rotations to get the global axes in local coords.
    # Luckily thats just the transposed in this case.
    def rotateGlob(self, deg, x, y, z):
        dx=x*self.transform[0] + y*self.transform[1] + z*self.transform[2]
        dy=x*self.transform[4] + y*self.transform[5] + z*self.transform[6]
        dz=x*self.transform[8] + y*self.transform[9] + z*self.transform[10]
        glMatrixMode(GL_MODELVIEW)
        glPushMatrix()
        glLoadMatrixf(self.transform)
        glRotatef(deg, dx,dy,dz)
        glGetFloatv(GL_MODELVIEW_MATRIX, self.transform)
        glPopMatrix()

