#!/usr/bin/env python

import sys
sys.path.append("../")

import pyglet
from pyglet import window
from pyglet.window import key
from maploader import svg2Buildings
from pyglet.gl import *
from math import tan, pi, cos, sin, acos, sqrt
from camera import Camera

windowWidth, windowHeight = 1024, 768

viewAngle = 45.0
fallspeed = 40.0

rotScale = 100.0
movScale = 100.0

periodMax = 1.0
periodMin = -1.0
periodMark = periodMin
jumpHeight = 2.0
jumpStep = 0.1
periodScale = 4.0

godMove = False

class MainWindow(window.Window):
    def __init__(self, *args, **kwargs):
        window.Window.__init__(self, *args, **kwargs)
        self.keys = key.KeyStateHandler()
        self.push_handlers(self.keys)
        self.buildings = svg2Buildings("game/map.svg", (1024, 1024))

        glEnable (GL_BLEND)
        glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_MULTISAMPLE_ARB)
        glEnable(GL_TEXTURE_2D)
        
        self.feather_tex = pyglet.image.load("game/feather_texture.png").get_texture()
        glBindTexture(self.feather_tex.target, self.feather_tex.id)
        glTexParameteri(self.feather_tex.target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE)
        glTexParameteri(self.feather_tex.target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE)

        self.heading = [1.0, 0.0, 0.0]
        glClearColor(0.0, 0.0, 0.0, 1.0)
        self.camera = Camera(100.0, 100.0, 256.0)
        r = -90.0
        self.camera.rotateLoc(r, 1.0, 0.0, 0.0)
        self.camera.rotateLoc(r, 0.0, 1.0, 0.0)
        self.period_move = -0.05
        
        self.posLabel = pyglet.text.Label("X: %s\tY: %s\tZ: %s" % (self.camera.transform[12], self.camera.transform[13], self.camera.transform[14]), color=(255, 255, 0, 255))
        
        self.leanLeft = False
        self.leanRight = False
        self.dive = False
        self.flatten = True
        self.backwards = False
        self.jumping = False
        self.jumpStage = 0.0
        
    def __del__(self):
        pass

    def draw_feather(self):

        glPushMatrix()

        glTranslatef(self.camera.transform[12] + 3, self.camera.transform[13], self.camera.transform[14] - 6)
        if self.jumping:
            if self.backwards:
                glRotatef(-180.0 * self.jumpStage, 0.0, 0.0, 1.0)
            else:
                glRotatef(180.0 * self.jumpStage, 0.0, 0.0, 1.0)
        else:
            if self.backwards:
                glRotatef(180.0, 0.0, 1.0, 0.0)
        
        if self.leanLeft and not self.leanRight:
            if self.backwards:
                glRotatef(-15.0, 1.0, 0.0, 0.0)
            else:
                glRotatef(15.0, 1.0, 0.0, 0.0)
            
        elif self.leanRight and not self.leanLeft:
            if self.backwards:
                glRotatef(15.0, 1.0, 0.0, 0.0)
            else:
                glRotatef(-15.0, 1.0, 0.0, 0.0)

        quad = gluNewQuadric()
        glColor3f(1.0, 1.0, 1.0)
        glBindTexture(self.feather_tex.target, self.feather_tex.id)
        gluQuadricNormals(quad, GLU_SMOOTH)
        gluQuadricTexture(quad, GL_TRUE)
        glScalef(5.0, 0.5, 1.0)
        gluDisk(quad, 0.0, 5.0, 20, 13)
        glBindTexture(self.feather_tex.target, 0)
        glPopMatrix()

    def draw_overlay(self):
        # switch us to ortho overlay 
        # thanks to http://www.lighthouse3d.com/opengl/glut/index.php?bmpfontortho
        glMatrixMode(GL_PROJECTION)
        glPushMatrix()
        glLoadIdentity()
        gluOrtho2D(0, windowWidth, 0, windowHeight)
        glMatrixMode(GL_MODELVIEW)

        glPushMatrix()
        glLoadIdentity()
        self.posLabel.text = "X: %.2f\tY: %.2f\tZ: %.2f" % (self.camera.transform[12], self.camera.transform[13], self.camera.transform[14])
        self.posLabel.draw()
        glPopMatrix()

        glMatrixMode(GL_PROJECTION)
        glPopMatrix()
        glMatrixMode(GL_MODELVIEW)

    def on_draw(self):
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

        # draw ground
        glPushMatrix()
        glLoadIdentity()
        self.camera.setView()

        # draw buildings
        for building in self.buildings:
            building.draw(self.collideCallback, self.hothouseCallback, (self.camera.transform[12], self.camera.transform[13], self.camera.transform[14]))

        self.draw_feather()
        glPopMatrix()

        self.draw_overlay()

    def collideCallback(self, building):
        #print "Collision!"
        pass
        
    def hothouseCallback(self, building):
        print "hothot"
        self.camera.transform[14] = 250.0
        
    def on_resize(self, w, h):
        glViewport(0, 0, w, h)

        fAspect = float(w) / float(h)

        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()

        gluPerspective(viewAngle, fAspect, 5.0, 3000.0)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()

    def jumpSpin(self):
        global periodMark
        self.jumping = True
        self.jumpStage = 1.0
        self.camera.moveGlob(0.0, 0.0, 1.0, jumpHeight)
        periodMark = periodMin
        self.period_move = -0.05
        
        
    def update(self, dt):
        global periodMark
        self.key_tick(dt)
        
        if self.jumping:
            self.jumpStage -= jumpStep
            if self.jumpStage <= 0.0:
                self.jumping = False
                self.jumpStage = 0.0
                self.backwards = not self.backwards

        # move based on current movement and key settings
        if self.flatten:
            # float back and forth
            if periodMark >= periodMax or periodMark <= periodMin:
                self.period_move = -self.period_move
            periodMark += self.period_move
                
            if periodMark < 0.0:
                self.camera.moveGlob(0.0, 0.0, -1.0, sqrt(0.125 * periodMark * periodMark) * periodScale + (self.jumpStage * jumpHeight))
            else:
                self.camera.moveGlob(0.0, 0.0, 1.0, sqrt(0.0625 * periodMark * periodMark) * periodScale + (self.jumpStage * jumpHeight))

            self.camera.moveGlob(self.heading[0], self.heading[1], self.heading[2], -periodMark * periodScale * 2)
        elif self.dive:
            # speed downward movement
            self.camera.moveGlob(0.0, 0.0, -1.0, fallspeed * dt)


    def key_tick(self, dt):
        self.leanLeft = False
        self.leanRight = False
        if godMove:
            if self.keys[key.DOWN]:
                    self.dive = True
                    self.flatten = False
                    self.camera.moveGlob(-1.0, 0.0, 0.0, movScale * dt)
            if self.keys[key.UP]:
                    self.flatten = True
                    self.dive = False
                    self.camera.moveGlob(1.0, 0.0, 0.0, movScale * dt)
            if self.keys[key.LEFT]:
                    self.leanLeft = True
                    self.camera.moveLoc(-1.0, 0.0, 0.0, movScale * dt)
            if self.keys[key.RIGHT]:
                    self.leanRight = True
                    self.camera.moveLoc(1.0, 0.0, 0.0, movScale * dt)
        else:
            if self.keys[key.DOWN]:
                    self.dive = True
                    self.flatten = False
            if self.keys[key.UP]:
                    self.flatten = True
                    self.dive = False
            if self.keys[key.LEFT]:
                    self.leanLeft = True
            if self.keys[key.RIGHT]:
                    self.leanRight = True
            
        if self.keys[key.O]:
            self.camera.moveGlob(0.0, 0.0, -1.0, movScale * dt)
        if self.keys[key.I]:
            self.camera.moveGlob(0.0, 0.0, 1.0, movScale * dt)

        if self.keys[key.S]:
            self.camera.rotateLoc(rotScale * dt, 1.0, 0.0, 0.0)
        if self.keys[key.A]:
            self.camera.rotateGlob(rotScale * dt, 0.0, 0.0, 1.0)
        if self.keys[key.D]:
            self.camera.rotateGlob(-rotScale * dt, 0.0, 0.0, 1.0)
        if self.keys[key.W]:
            self.camera.rotateLoc(-rotScale * dt, 1.0, 0.0, 0.0)
            
        if self.leanLeft and not self.leanRight:
            self.heading[1] = -0.5
        elif self.leanRight and not self.leanLeft:
            self.heading[1] = 0.5
        else:
            self.heading[1] = 0.0
            
    def on_key_press(self, symbol, modifier):
        if symbol == key.SCROLLLOCK:
            pyglet.image.get_buffer_manager().get_color_buffer().save('screenshot.png')
        if symbol == key.SPACE:
            self.jumpSpin()
        if symbol == key.ESCAPE:
            self.on_close()
            
if __name__ == '__main__':
    w = MainWindow(windowWidth, windowHeight, caption='Feather Surfer', resizable=True)
    pyglet.clock.schedule_interval(w.update, 1.0/20.0)
    pyglet.app.run()
