#!/usr/bin/env python

import xml.dom.minidom
from buildingstuff import Building

def svg2Buildings(filename, dims):
    buildings = list()
    
    d = xml.dom.minidom.parse(filename)
    root = d.getElementsByTagName('svg')[0]
    
    xscale = float(dims[0]) / float(root.attributes['width'].value)
    yscale = float(dims[1]) / float(root.attributes['height'].value)
    
    domnodes = d.getElementsByTagName('rect')
    for rect in domnodes:
        dimdict = dict()
        
        # should check for map boundaries here
        dimdict['x'] = int(float(rect.attributes['x'].value) * xscale + 0.5)
        dimdict['y'] = int(float(rect.attributes['y'].value) * yscale + 0.5)
        dimdict['width'] = int(float(rect.attributes['width'].value) * xscale + 0.5)
        dimdict['height'] = int(float(rect.attributes['height'].value) * yscale + 0.5)
        
        # this depends on the order of things in the style attribute, not very tolerant
        for field in rect.attributes['style'].value.split(';'):
            if field.startswith('fill:'):
                dimdict['depth'] = int(field.split(':#')[1][0:2], 16)
        buildings.append(Building(dimdict))
        
    return buildings

if __name__ == '__main__':
    
    buildings = svg2Buildings("map.svg", (1024, 768))

